autosave
' Black Jack for the PicoMiteVGA
' Version 0.2
' by Vegipete, May, 2023
'
' User input via console.
' [SPACE] to hit, [ENTER] to stand, [ESC] to quit
'
' Should work on other 'mites with a display that allow transparent text.
'
' Quick implementation of BlackJack to test playing cards as fonts.
' No extended rules such as doubling down or splits.
' Not even any tracking of winnings, at this stage.
'
' Uses a single deck. Cards are reshuffled when 12 or fewer remain.
'
' The last subroutine and the font at the end of this program are all
' that is required to display desired playing cards. More colour
' refinement is possible, along with jokers.
'
Option default integer
MODE 2
CLS
Randomize Timer

Dim deck(52)
Dim vcards(13) = (0,11,2,3,4,5,6,7,8,9,10,10,10,10) ' card values
Dim dcards(21)  ' max 8 cards in hand
Dim pcards(21)  ' max 8 cards in hand
Dim cc,dl,pl

ShuffleCards
Do
  dl = 0  ' dealer card count
  Math set 0,dcards()
  pl = 0  ' player card count
  Math set 0,pcards()

  CLS

  Inc cc
  ShowCard(deck(cc),60,150) ' first player card
  pcards(pl) = deck(cc)
  Inc pl

  Inc cc
  ShowCard(0,60,50)   ' first dealer card, face down
  dcards(dl) = deck(cc)
  Inc dl

  Inc cc
  ShowCard(deck(cc),60 + pl * 40,150) ' second player card
  pcards(pl) = deck(cc)
  Inc pl

  Inc cc
  ShowCard(deck(cc),60 + dl * 40,50)  ' second dealer card, visible
  dcards(dl) = deck(cc)
  Inc dl

  If CardSum(pcards()) = 21 Then    ' player has 21
    If CardSum(dcards()) = 21 Then  ' dealer also has 21
      ShowResult("Push. No winner.")
    Else
      ShowResult("BLACKJACK! Player wins!")
    EndIf
  Else  ' player might want to hit
    Text 100,205,"[SPACE] to hit, [ENTER] to stand.",CM
    Do
      kp = Asc(Inkey$)
      If kp = 32 Then ' hit
        Inc cc
        If cc > 52 Then cc = 1  ' roll the deck - bad idea!
        ShowCard(deck(cc),60 + pl * 40,150) ' next player card
        pcards(pl) = deck(cc)
        Inc pl

        If CardSum(pcards()) > 21 Then  ' player bust
          ShowResult("Bust. Player loses.")
          Exit ' new hand
        EndIf
      ElseIf kp = 13 Then ' stay, now dealer can finish
        Do While CardSum(dcards()) < 17
          Inc cc
          If cc > 52 Then cc = 1  ' roll the deck - bad idea!
          ShowCard(deck(cc),60 + dl * 40,50)  ' next dealer card, visible
          dcards(dl) = deck(cc)
          Inc dl
        Loop
        If CardSum(dcards()) > 21 Then
          ShowResult("Dealer busts. Player wins!")
        ElseIf CardSum(dcards()) = CardSum(pcards()) Then
          ShowResult("Push. No winner.")
        ElseIf CardSum(dcards()) > CardSum(pcards()) Then
          If CardSum(dcards()) = 21 And dl = 2 Then
            ShowResult("BLACKJACK! Dealer wins.")
          Else
            ShowResult("Dealer wins.")
          EndIf
        Else
          ShowResult("Player wins.")
        EndIf
        Exit ' new hand
      ElseIf kp = 27 Then
        QuitGame
      EndIf
    Loop
  EndIf

  Do
    kp = Asc(Inkey$)
    If kp = 27 Then QuitGame
    If kp = 32 Then Exit ' space bar for another hand
  Loop

  If cc > 40 Then ShuffleCards

Loop

' Game over, quit
Sub QuitGame
  MODE 1
  CLS
  Print "Good bye!"
  End
End Sub

' Show the result of the hand
Sub ShowResult(r$)
  ShowCard(dcards(0),60,50)   ' turn over hidden card
  Text 100,205,Space$(33),CM  ' erase message (if showing)
  Text 100,205,"[SPACE] to play again.",CM
  Text 110,125,r$,CM
End Sub

' Shuffle the deck
Sub ShuffleCards
  Local i,t
  For i = 1 To 52   ' initialize the deck
    deck(i) = i
  Next i
  For i = 1 To 52   ' shuffle the deck
    t = 1 + Int(Rnd * 52)
    Swap(deck(i), deck(t)))
  Next i

  cc = 0  ' position in deck

End Sub

' calculate the value of the hand
' Aces are worth 11 unless the total is over 21
Function CardSum(c())
  Local i,a,s,x
  a = 0   ' no ace to start
  s = 0   ' sum = 0
  For i = 0 To 8
    x = (c(i)-1) Mod 13 + 1
    Inc s, vcards(x)
    If x = 1 Then Inc a   ' this card is an ace
  Next i
  Do While s > 21 And a > 0
    Inc s, -10
    Inc a, -1
  Loop
  CardSum = s
End Function

' Swap two numbers
Sub Swap(a,b)
  Local t
  t = a : a = b : b = t
End Sub

'===================================================================
' Show a particular card on screen
' x,y is location (top left corner of card)
' n = card number, 1 to 52
'     0 = face down
'     1-13 = spade, 14-26 = club, 27-39 = diamond, 40-52 = heart
Sub ShowCard(n,x,y)
  Local c1,c2,c3,suit,v
  If n < 1 Then   ' show back of card
    Text x,y,"0",LT,8,1,RGB(red),RGB(black)
    Exit Sub
  ElseIf n > 52 Then
    ' error - illegal card number
    ' may want to show Joker for card number 53/54
    Exit Sub
  EndIf

  v = 1 + (n-1) Mod 13
  suit = (n-1) \ 13
  If suit = 0 Or suit = 1 Then
    c1 = RGB(blue)
    c2 = RGB(cyan)
    c3 = RGB(black)
  ElseIf suit = 2 Or suit = 3 Then
    c1 = RGB(red)
    c2 = RGB(brown)
    c3 = RGB(red)
  Else
    ' error - illegal suit
    Exit Sub
  EndIf

  Text x,y,"1",LT,8,1,c1,RGB(black)   ' frame
  Text x,y,"2",LT,8,1,RGB(white),-1   ' background
  Text x,y,"@",LT,8,1,c2,-1           ' frame hilite
  If v > 10 Then
    Text x,y,"N",LT,8,1,RGB(yellow),-1 ' yellow for crown
  EndIf
  Text x,y,Chr$(64+v),LT,8,1,c3,-1    ' number
  If v = 1 Then
    Text x,y,Chr$(55+suit),LT,8,1,c1,-1 ' ace glyph
  Else
    Text x,y,Chr$(51+suit),LT,8,1,c1,-1 ' suit glyph
  EndIf

End Sub

' Play.bas
' Font type    : Numeric (32 characters)
' Font start   : CHR$(48)
' Font size    : 32x48 pixels
' Memory usage : 6148 bytes
DefineFont #8
  20303020
  00000000 FCFFFF3F 06000060 F23FFC4F 0A499250 2A499254 4A499252 8A481251
  7A44225E 0A442250 0A424250 8A418251 7A581A5E 0A4C3250 CA4FF253 EA47E257
  EA23C457 EA118857 EA881157 4A842152 0A424250 0AA18550 F2500A4F 02AC3540
  02AC3540 F2500A4F 0AA18550 0A424250 4A842152 EA881157 EA118857 EA23C457
  EA47E257 CA4FF253 0A4C3250 7A581A5E 8A418251 0A424250 0A442250 7A44225E
  8A481251 4A499252 2A499254 0A499250 F23FFC4F 06000060 FCFFFF3F 00000000
  00000000 FCFFFF3F 06000060 02000040 02000040 02000040 02000040 02000040
  02000040 02000040 02000040 02000040 02000040 02000040 02000040 02000040
  02000040 02000040 02000040 02000040 02000040 02000040 02000040 02000040
  02000040 02000040 02000040 02000040 02000040 02000040 02000040 02000040
  02000040 02000040 02000040 02000040 02000040 02000040 02000040 02000040
  02000040 02000040 02000040 02000040 02000040 06000060 FCFFFF3F 00000000
  00000000 00000000 F8FFFF1F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F
  FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F
  FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F
  FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F
  FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F
  FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F FCFFFF3F F8FFFF1F 00000000 00000000
  00000000 00000000 00000000 80010000 C0030000 E0070000 F00F0000 F81F0000
  F81F0000 F81F0000 B00D0000 C0030000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 0000C003 0000B00D 0000F81F 0000F81F
  0000F81F 0000F00F 0000E007 0000C003 00008001 00000000 00000000 00000000
  00000000 00000000 00000000 80010000 C0030000 C0030000 B00D0000 F81F0000
  F81F0000 B00D0000 80010000 C0030000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 0000C003 00008001 0000B00D 0000F81F
  0000F81F 0000B00D 0000C003 0000C003 00008001 00000000 00000000 00000000
  00000000 00000000 00000000 80000000 C0010000 E0030000 F0070000 F80F0000
  F0070000 E0030000 C0010000 80000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000001 00008003 0000C007 0000E00F
  0000F01F 0000E00F 0000C007 00008003 00000001 00000000 00000000 00000000
  00000000 00000000 00000000 700E0000 F81F0000 F81F0000 F81F0000 F81F0000
  F00F0000 E0070000 C0030000 80010000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00008001 0000C003 0000E007 0000F00F
  0000F81F 0000F81F 0000F81F 0000F81F 0000700E 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00800100 00400300 00A00700
  00D00F00 00E81F00 00F43F00 00FA7F00 00FFFF00 00FFFF00 80FFFF01 80FFFF01
  80FFFF01 80FFFF01 80FFFF01 80FFFF01 00BFFD00 009E7900 00800100 00800100
  00800100 00C00300 00E00700 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00C00300 00A00700 00D00F00
  00F00F00 00F00F00 00F00F00 00F00F00 00E00700 00800100 009E7900 00BDF500
  80FEFB01 80FFFF01 80FFFF01 80FFFF01 00BFFD00 009E7900 00800100 00800100
  00800100 00C00300 00E00700 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00800100 00400300
  00A00700 00D00F00 00E81F00 00F43F00 00FA7F00 00FDFF00 80FEFF01 40FFFF03
  C0FFFF03 80FFFF01 00FFFF00 00FE7F00 00FC3F00 00F81F00 00F00F00 00E00700
  00C00300 00800100 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 001C3800
  003A7400 007DFA00 80FEFD01 80FFFF01 80FFFF01 80FFFF01 80FFFF01 80FFFF01
  80FFFF01 00FFFF00 00FFFF00 00FE7F00 00FE7F00 00FC3F00 00F81F00 00F00F00
  00E00700 00C00300 00800100 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 10000008 F8EE771F 10000008 10000008 10000008
  00000000 10000008 10000008 10000008 00000000 10000008 10000008 10000008
  10000008 10000008 00000000 10000008 00000000 10000008 10000008 10000008
  10000008 10000008 10000008 00000000 10000008 00000000 10000008 10000008
  10000008 10000008 10000008 00000000 10000008 10000008 10000008 00000000
  10000008 10000008 10000008 F8EE771F 10000008 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000003 00008004 00008004
  00008004 00008007 00008004 00008004 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 20010000 20010000 E0010000 20010000
  20010000 20010000 C0000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000003 00008004 00008000
  00000001 00000002 00000004 00008007 00000000 00000000 00000000 00000000
  00000000 00800100 00C00300 00C00300 00800100 00000000 00000000 00000000
  00000000 00000000 00000000 00800100 00C00300 00C00300 00800100 00000000
  00000000 00000000 00000000 00000000 E0010000 20000000 40000000 80000000
  00010000 20010000 C0000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000003 00008004 00008000
  00000001 00008000 00008004 00000003 00000000 00000000 00000000 00000000
  00800100 00C00300 00C00300 00800100 00000000 00000000 00800100 00C00300
  00C00300 00800100 00000000 00000000 00800100 00C00300 00C00300 00800100
  00000000 00000000 00000000 00000000 C0000000 20010000 00010000 80000000
  00010000 20010000 C0000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000001 00000003 00000005
  00000005 00008007 00000001 00000001 00000000 00000000 00000000 00000000
  00000000 00000000 00181800 003C3C00 003C3C00 00181800 00000000 00000000
  00000000 00000000 00181800 003C3C00 003C3C00 00181800 00000000 00000000
  00000000 00000000 00000000 00000000 80000000 80000000 E0010000 A0000000
  A0000000 C0000000 80000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00008007 00000004 00000007
  00008000 00008000 00008004 00000003 00000000 00000000 00000000 00000000
  00000000 000C3000 001E7800 001E7800 000C3000 00000000 00800100 00C00300
  00C00300 00800100 00000000 000C3000 001E7800 001E7800 000C3000 00000000
  00000000 00000000 00000000 00000000 C0000000 20010000 00010000 00010000
  E0000000 20000000 E0010000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000001 00000002 00000004
  00000007 00008004 00008004 00000003 00000000 00000000 00000000 00000000
  00181800 003C3C00 003C3C00 00181800 00000000 00000000 00181800 003C3C00
  003C3C00 00181800 00000000 00000000 00181800 003C3C00 003C3C00 00181800
  00000000 00000000 00000000 00000000 C0000000 20010000 20010000 E0000000
  20000000 40000000 80000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00008007 00008000 00008000
  00000001 00000002 00000002 00000002 00000000 00000000 00000000 00000000
  00181800 003C3C00 003C3C00 00181800 00000000 00000000 00866100 00CFF300
  00CFF300 00866100 00000000 00000000 00181800 003C3C00 003C3C00 00181800
  00000000 00000000 00000000 00000000 80000000 80000000 80000000 40000000
  20000000 20000000 E0010000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000003 00008004 00008004
  00000003 00008004 00008004 00000003 00000000 00181800 003C3C00 003C3C00
  00181800 00000000 00000000 00181800 003C3C00 003C3C00 00181800 00000000
  00000000 00181800 003C3C00 003C3C00 00181800 00000000 00000000 00181800
  003C3C00 003C3C00 00181800 00000000 C0000000 20010000 20010000 C0000000
  20010000 20010000 C0000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000003 00008004 00008004
  00008003 00008000 00000001 00000002 00000000 00300C00 00781E00 00781E00
  00300C00 00000000 00000000 00066000 000FF000 000FF000 00866100 00C00300
  00C00300 00866100 000FF000 000FF000 00066000 00000000 00000000 00300C00
  00781E00 00781E00 00300C00 00000000 40000000 80000000 00010000 C0010000
  20010000 20010000 C0000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00006002 00009006 00009002
  00009002 00009002 00009002 00006002 00000000 00066000 000FF000 000FF000
  00866100 00C00300 00C00300 00866100 000FF000 000FF000 00066000 00000000
  00000000 00066000 000FF000 000FF000 00866100 00C00300 00C00300 00866100
  000FF000 000FF000 00066000 00000000 40060000 40090000 40090000 40090000
  40090000 60090000 40060000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 0000C001 00008000 00008000
  00008000 00008000 00008004 00000003 00000000 00000000 00000000 00000000
  00400500 00A00A00 00200800 00200800 00E00F00 00E00800 00F01000 00F82400
  00F82000 00F83800 00780C00 00780800 007C0600 00FC0300 00FC0F00 00F01F00
  00000000 00000000 00000000 00000000 C0000000 20010000 00010000 00010000
  00010000 00010000 80030000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00008003 00004004 00004004
  00004004 00004004 00008004 00004003 00000000 00000000 00000000 00000000
  00A00A00 00600D00 00200800 00200800 00E00F00 00E00900 00F01000 00781400
  00382000 00383800 00780600 007C0400 00FC0400 00FE0300 00FE0F00 00FE1F00
  000E0000 000E0000 00070000 00000000 C0020000 20010000 20020000 20020000
  20020000 20020000 C0010000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00004004 00008004 00000005
  00000006 00000005 00008004 00004004 00000000 00000000 00000000 00101000
  00B01A00 00600D00 00200800 00200800 00E00F00 00E00800 00E01000 00702400
  00702000 00703800 00780C00 00380800 003C0E00 003C0200 00FC1F00 00FE1F00
  00000000 00000000 00000000 00000000 20020000 20010000 A0000000 60000000
  A0000000 20010000 20020000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00C00700 00C00700 00C00700 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
  00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
End DefineFont
